#!/usr/bin/env python
from datetime import date, timedelta
from slugify import slugify
import sys
from autolink import linkify
from dateutil.relativedelta import relativedelta
import re
from pathlib import Path
from locale import setlocale, LC_TIME

NEWLINE = "\n"

here = Path(__file__).parent
setlocale(LC_TIME, 'fr_FR.utf8')
previous_month = f"{(date.today() - relativedelta(months=1)):%B}"

template = lambda content: (here / "template.mjml").read_text().replace('%content%', content).replace('%year%', str(date.today().year)).replace('%month%', previous_month)

def codeify(content: str) -> str:
    inside_code = False
    out = ""
    for char in content:
        if char == "`":
            out += "</code>" if inside_code else "<code>"
            inside_code = not inside_code
        else:
            out += char 

    return out

def mj_section(title: str, content: str) -> str:
    h2 = f"<h2>{title}</h2>" if title else ""

    image_filename = f"{date.today().year}/{previous_month}-attachments/{slugify(title, only_ascii=True)}.png"

    image = f'<mj-image src="https://git.inpt.fr/net7/mailinkid/-/raw/main/{image_filename}" />' if (here / image_filename).exists() else ""

    return f"""
<mj-section>
    <mj-column>
        {image}
        <mj-text>
            {h2}
            <p>{codeify(linkify(content)).replace(NEWLINE*2, NEWLINE).replace(NEWLINE, '</p><p>')}</p>
        </mj-text>
    </mj-column>
</mj-section>
    """

def text_to_mjml(content: str) -> str:
    before_first_section = True
    intro = ""
    post_scriptum = ""
    inside_ps = False

    # type: Array<{ title: string; content: string }>
    sections = []
    current_section = {
            "title" :"",
            "content": ""
            }

    for line in content.splitlines():
        if inside_ps:
            post_scriptum += f"{line}\n"
        elif line.startswith('*'):
            if current_section["title"]:
                sections.append(current_section)
            current_section = {
                    "title": line.lstrip("*"),
                    "content" :""
            }
            before_first_section = False
        elif before_first_section:
            intro += f"{line}\n"
        else:
            if line.lower().startswith("ps : ") or line.lower().startswith("ps: "):
                inside_ps = True
                post_scriptum += f"{line}\n"
            else:
                current_section["content"] += f"{line}\n"

    sections.append(current_section)

    return mj_section(title="", content=intro) + "".join(mj_section(**section) for section in sections) + mj_section(title="", content=post_scriptum)


outpath = (here / str(date.today().year) / f'{previous_month}.mjml')
outpath.write_text(template(text_to_mjml(sys.stdin.read())))
print(outpath.resolve())
